@@ -1,10 +1,12 @@
+from datetime import datetime
+import ssl
+import asyncio
+
 import aiohttp
-from nonebot import get_bot, get_driver
+from nonebot import get_bot, get_driver, require
 from nonebot.log import logger
 from nonebot.adapters.onebot.v11 import MessageSegment, Bot
-from datetime import datetime
-import asyncio
-import ssl
+
 from .db_action import (
     load_last_processed,
     save_last_processed,
@@ -12,28 +14,40 @@ from .db_action import (
     change_group_repo_cfg,
 )
 from .config import config
+
+# Ensure required plugins are loaded
+require("nonebot_plugin_htmlrender")
+# pylint: disable=wrong-import-position 
+from nonebot_plugin_htmlrender import html_to_pic
+
 superusers = get_driver().config.superusers
 
 # Load GitHub token from environment variables
 GITHUB_TOKEN: str | None = config.github_token
-max_retries: int = config.github_validate_retries
-delay: int = config.github_validate_delay
+max_retries: int = config.github_retries
+delay: int = config.github_retry_delay
+
+# Global cache to store API responses
+api_cache = {}
 
 default_sending_templates = {
     "commit": "📜 New Commit in {repo}\n\n"
-              "Message: {message}\nAuthor: {author}\nURL: {url}",
+              "Message: {message}\nAuthor: {author}\n"
+              "Commit time: {time}\nURL: {url}",
     "issue": "🐛 **New Issue in {repo}!**\n\n"
-             "Title: {title}\nAuthor: {author}\nURL: {url}",
+             "Title: {title}\nAuthor: {author}\n"
+             "Issue created time: {time}\nURL: {url}",
     "pull_req": "🔀 **New Pull Request in {repo}!**\n\n"
-                "Title: {title}\nAuthor: {author}\nURL: {url}",
+                "Title: {title}\nAuthor: {author}\n"
+                "Pr created at: {time}\nURL: {url}",
     "release": "🚀 **New Release for {repo}!**\n\n"
                "**Name:** {name}\nVersion: {version}\n"
-               "Details:\n {details}\nURL: {url}",
+               "Details:\n {details}\nRelease time:{time}\nURL: {url}",
 }
 config_template = config.github_sending_templates
 
 
-async def validate_github_token(retries=3, delay=5) -> bool:
+async def validate_github_token(retries=3, retry_delay=5) -> bool:
     """
     Validate the GitHub token by making a test request,
     with retries on SSL errors.
@@ -46,7 +60,7 @@ async def validate_github_token(retries=3, delay=5) -> bool:
         return False
 
     headers = {"Authorization": f"token {GITHUB_TOKEN}"}
-    for attempt in range(retries):
+    for _ in range(retries):
         try:
             async with aiohttp.ClientSession() as session:
                 async with session.get(
@@ -55,20 +69,20 @@ async def validate_github_token(retries=3, delay=5) -> bool:
                     if response.status == 200:
                         logger.info("GitHub token is valid.")
                         return True
-                    else:
-                        logger.error(
-                                    f"GitHub token validation failed: "
-                                    f"{response.status} - "
-                                    f"{await response.text()}"
-                                    )
-                        GITHUB_TOKEN = None
-                        return False
+                    
+                    logger.error(
+                                f"GitHub token validation failed: "
+                                f"{response.status} - "
+                                f"{await response.text()}"
+                                )
+                    GITHUB_TOKEN = None
+                    return False
         except ssl.SSLError as e:
             logger.error(
                 f"SSL error during GitHub token validation: {e}. "
-                f"Retrying in {delay} seconds..."
+                f"Retrying in {retry_delay} seconds..."
             )
-            await asyncio.sleep(delay)
+            await asyncio.sleep(retry_delay)
         except Exception as e:
             logger.error(f"Error validating GitHub token: {e}")
 
@@ -77,8 +91,21 @@ async def validate_github_token(retries=3, delay=5) -> bool:
     return False
 
 
-async def fetch_github_data(repo: str, endpoint: str) -> dict | None:
-    """Fetch data from the GitHub API for a specific repo and endpoint."""
+async def fetch_github_data(repo: str, endpoint: str) -> list | None:
+    """
+    Fetch data from the GitHub API
+    for a specific repo and endpoint.
+    """
+    cache = api_cache.get(repo, {}).get(endpoint, [])
+
+    # Check if the data exists in the cache
+    if cache:
+        logger.debug(f'Using cached data for {repo}/{endpoint}')
+        if cache[0] == []:
+            return []
+        return cache
+
+    # If not in cache, fetch from the API
     api_url = f"https://api.github.com/repos/{repo}/{endpoint}"
     headers = (
         {"Authorization": f"token {GITHUB_TOKEN}"}
@@ -86,32 +113,40 @@ async def fetch_github_data(repo: str, endpoint: str) -> dict | None:
         else {}
     )
 
-    try:
-        async with aiohttp.ClientSession() as session:
-            async with session.get(api_url, headers=headers) as response:
-                response.raise_for_status()
-                return await response.json()
-    except aiohttp.ClientResponseError as e:
-        logger.error(
-            f"HTTP error while fetching GitHub {endpoint} for {repo}: {e}"
-        )
-        return {
-            "falt": f"Failed to fetch GitHub {endpoint} for {repo}: {e}"
-        }
-    except Exception as e:
-        logger.opt(exception=e).error(repr(e))
-        logger.error(
-            f"Unexpected error while fetching GitHub "
-            f"{endpoint} for {repo}: "
-            f"{e}"
-        )
-        return {
-            "falt": (
+    retries = 1
+    errs = []
+    while retries <= max_retries:
+        try:
+            async with aiohttp.ClientSession() as session:
+                async with session.get(api_url, headers=headers) as response:
+                    response.raise_for_status()
+                    data = await response.json()
+                    # Cache the response
+                    repo_data = api_cache.get(repo, {})
+                    repo_data[endpoint] = data if data else [[]]
+                    api_cache[repo] = repo_data
+                    return data
+        except aiohttp.ClientResponseError as e:
+            logger.error(
+                f"HTTP error while fetching GitHub {endpoint} for {repo} "
+                f"in attempt {retries}: {e}"
+            )
+            errs.append(f'{e.__class__.__name__}: {e}')
+        except Exception as e:
+            logger.error(
                 f"Unexpected error while fetching GitHub "
-                f"{endpoint} for {repo}: "
+                f"{endpoint} for {repo} in attempt {retries}: "
                 f"{e}"
             )
-        }
+            errs.append(f'{e.__class__.__name__}: {e}\nargs:{e.args}')
+        await asyncio.sleep(delay)
+        retries += 1
+
+    return {
+        "falt": f"Failed to fetch {endpoint} for {repo} after "
+                f"{max_retries} attempts.",
+        'errors': errs,
+    }
 
 
 async def notify(
@@ -140,9 +175,12 @@ async def notify(
         ):
             message = format_message(repo, item, data_type)
             try:
-                await bot.send_group_msg(
-                    group_id=group_id, message=MessageSegment.text(message)
-                )
+                if 'issue' == data_type and 'pull' in message:
+                    pass
+                else:
+                    await bot.send_group_msg(
+                        group_id=group_id, message=MessageSegment.text(message)
+                    )
             except Exception as e:
                 logger.error(
                         f"Failed to notify group {group_id} about {data_type} "
@@ -169,6 +207,7 @@ def format_message(repo: str, item: dict, data_type: str) -> str:
             "message": item["commit"]["message"],
             "author": item["commit"]["committer"]["name"],
             "url": item["html_url"],
+            "time": item["commit"]["committer"]["date"],
         }
     elif data_type == "issue":
         datas = {
@@ -176,6 +215,7 @@ def format_message(repo: str, item: dict, data_type: str) -> str:
             "title": item["title"],
             "author": item["user"]["login"],
             "url": item["html_url"],
+            "time": item["created_at"],
         }
     elif data_type == "pull_req":
         datas = {
@@ -183,6 +223,7 @@ def format_message(repo: str, item: dict, data_type: str) -> str:
             "title": item["title"],
             "author": item["user"]["login"],
             "url": item["html_url"],
+            "time": item["created_at"],
         }
     elif data_type == "release":
         datas = {
@@ -191,6 +232,7 @@ def format_message(repo: str, item: dict, data_type: str) -> str:
             "version": item.get("tag_name", "Unknown Version"),
             "details": item.get("body", "No description provided."),
             "url": item.get("html_url", "No URL"),
+            "time": item.get("published_at", "Unknown time"),
         }
     else:
         return "Unknown data type."
@@ -205,9 +247,15 @@ async def check_repo_updates():
     Check for new commits, issues, PRs, and releases for all repos
     and notify groups.
     """
-    bot: Bot = get_bot()
-    last_processed = load_last_processed()
-    group_repo_dict = load_groups()
+    global api_cache
+    api_cache = {}
+    try:
+        bot: Bot = get_bot()
+        last_processed = load_last_processed()
+        group_repo_dict = load_groups()
+    except Exception:
+        return
+
     for group_id, repo_configs in group_repo_dict.items():
         group_id = int(group_id)
         for repo_config in repo_configs:
@@ -219,13 +267,14 @@ async def check_repo_updates():
                 ("release", "releases"),
             ]:
                 if repo_config.get(data_type, False):
+                    # Fetch data (use cache if available)
                     data = await fetch_github_data(repo, endpoint)
                     if "falt" not in data and data:
                         await notify(
                             bot,
                             group_id,
                             repo,
-                            data if isinstance(data, list) else [data],
+                            data,
                             data_type,
                             last_processed,
                         )
@@ -233,10 +282,33 @@ async def check_repo_updates():
                         logger.error(data["falt"])
                         if config.github_send_faliure_group:
                             try:
-                                await bot.send_group_msg(
-                                    group_id=group_id,
-                                    message=data["falt"]
-                                )
+                                if any('403' in x for x in data["errors"]):
+                                    await bot.send_group_msg(
+                                        group_id=group_id,
+                                        message=(
+                                            MessageSegment.text(
+                                                data['falt'] + "\nGitHub API rate limit exceeded.\n Probably caused by invalid / no token."
+                                            )
+                                        )
+                                    )
+                                else:
+                                    html = '<p>GitHub API Error:</p>'
+                                    html += "".join(
+                                        (
+                                            "<p style='white-space=pre-wrap'>"
+                                            + x.replace("\n", "<br>")
+                                            + "</p>"
+                                        )
+                                        for x in data["errors"]
+                                    )
+                                    pic = await html_to_pic(html)
+                                    await bot.send_group_msg(
+                                        group_id=group_id,
+                                        message=(
+                                            MessageSegment.text(data["falt"])
+                                            + MessageSegment.image(pic)
+                                        )
+                                    )
                             except Exception as e:
                                 logger.error(
                                         f"Failed to notify group {group_id} "
@@ -245,19 +317,41 @@ async def check_repo_updates():
                         if config.github_send_faliure_superuser:
                             for users in superusers:
                                 try:
-                                    await bot.send_private_msg(
-                                        user_id=users,
-                                        message=data["falt"]
-                                    )
+                                    if any('403' in x for x in data["errors"]):
+                                        await bot.send_private_msg(
+                                            user_id=users,
+                                            message=(
+                                                MessageSegment.text(
+                                                    data['falt'] + "\nGitHub API rate limit exceeded.\n Probably caused by invalid / no token."
+                                                )
+                                            )
+                                        )
+                                    else:
+                                        html = '<p>GitHub API Error:</p>'
+                                        html += "".join(
+                                            "<p style='white-space=pre-wrap'>"
+                                            + x.replace("\n", "<br>")
+                                            + "</p>"
+                                            for x in data["errors"]
+                                        )
+                                        pic = await html_to_pic(html)
+                                        await bot.send_private_msg(
+                                            user_id=users,
+                                            message=(
+                                                MessageSegment.text(data["falt"])
+                                                + MessageSegment.image(pic)
+                                            )
+                                        )
                                 except Exception as e:
                                     logger.error(
-                                        "Failed to notify the superuser"
+                                        "Failed to notify the superuser "
                                         "about the error: "
                                         f"{e}"
                                     )
                         if (
                             config.github_disable_when_fail
                             and "SSL" not in data["falt"]
+                            and "403" not in data["falt"]
                         ):
                             repo_config[data_type] = False
                             change_group_repo_cfg(
